<?php

namespace Oro\Bundle\ProductBundle\Tests\Unit\EventListener;

use Oro\Bundle\DataGridBundle\Datagrid\ParameterBag;
use Oro\Bundle\DataGridBundle\Datagrid\RequestParameterBagFactory;
use Oro\Bundle\ProductBundle\Event\ProductGridWidgetRenderEvent;
use Oro\Bundle\ProductBundle\EventListener\ProductGridWidgetRenderEventListener;

class ProductGridWidgetRenderEventListenerTest extends \PHPUnit\Framework\TestCase
{
    /** @var ProductGridWidgetRenderEventListener */
    protected $listener;

    /** @var \PHPUnit\Framework\MockObject\MockObject|RequestParameterBagFactory */
    protected $requestParameterBagFactory;

    protected function setUp(): void
    {
        $this->requestParameterBagFactory = $this
            ->getMockBuilder('Oro\Bundle\DataGridBundle\Datagrid\RequestParameterBagFactory')
            ->disableOriginalConstructor()
            ->getMock();

        $this->listener = new ProductGridWidgetRenderEventListener($this->requestParameterBagFactory);
    }

    protected function tearDown(): void
    {
        unset($this->listener, $this->requestParameterBagFactory);
    }

    public function testOnWidgetRender()
    {
        $gridParams = ['i' => 1, 'p' => 25, 's' => []];
        $event = new ProductGridWidgetRenderEvent(['grid' => ['removed' => 'params']]);

        $this->requestParameterBagFactory->expects($this->once())->method('createParameters')
            ->willReturn(new ParameterBag($gridParams));
        $this->listener->onWidgetRender($event);

        $eventParams = $event->getWidgetRouteParameters();
        $this->assertArrayHasKey('grid', $eventParams);
        $this->assertSame($gridParams, $eventParams['grid']);
    }
}
